@@ -1,5 +1,5 @@
 /*
- * peas-extension.h
+ * peas-extension.c
  * This file is part of libpeas
  *
  * Copyright (C) 2010 Steve Frécinaux
@@ -24,6 +24,7 @@
 #endif
 
 #include "peas-extension.h"
+#include "peas-extension-priv.h"
 #include "peas-introspection.h"
 
 /**
@@ -37,7 +38,7 @@
  * loaded plugins.
  *
  * To properly use the proxy instances, you will need GObject-introspection
- * data for the #GInterface or #GObjectClass you want to use as extension
+ * data for the #GInterface or #GObjectClass you want to use as an extension
  * point.  For instance, if you wish to use #PeasActivatable, you will need to
  * put the following code excerpt in the engine initialization code, in order
  * to load the required "Peas" typelib:
@@ -69,10 +70,6 @@ enum {
   PROP_EXTENSION_TYPE
 };
 
-struct _PeasExtensionPrivate {
-  GType exten_type;
-};
-
 static void
 peas_extension_init (PeasExtension *exten)
 {
@@ -119,6 +116,17 @@ peas_extension_get_property (GObject    *object,
     }
 }
 
+static void
+peas_extension_constructed (GObject *object)
+{
+  PeasExtension *exten = PEAS_EXTENSION (object);
+
+  exten->priv->constructed = TRUE;
+
+  if (G_OBJECT_CLASS (peas_extension_parent_class)->constructed != NULL)
+    G_OBJECT_CLASS (peas_extension_parent_class)->constructed (object);
+}
+
 static void
 peas_extension_class_init (PeasExtensionClass *klass)
 {
@@ -126,11 +134,17 @@ peas_extension_class_init (PeasExtensionClass *klass)
 
   object_class->set_property = peas_extension_set_property;
   object_class->get_property = peas_extension_get_property;
+  object_class->constructed = peas_extension_constructed;
 
+  /**
+   * PeasExtension:extension-type:
+   *
+   * The GType of the interface being proxied.
+   */
   g_object_class_install_property (object_class, PROP_EXTENSION_TYPE,
                                    g_param_spec_gtype ("extension-type",
                                                        "Extension Type",
-                                                       "The GType of this extesion",
+                                                       "The GType of the interface being proxied",
                                                        G_TYPE_NONE,
                                                        G_PARAM_READWRITE |
                                                        G_PARAM_CONSTRUCT_ONLY |
@@ -156,7 +170,7 @@ peas_extension_get_extension_type (PeasExtension *exten)
 }
 
 /**
- * peas_extension_call:
+ * peas_extension_call: (skip)
  * @exten: A #PeasExtension.
  * @method_name: the name of the method that should be called.
  * @Varargs: arguments for the method.
@@ -195,7 +209,7 @@ peas_extension_call (PeasExtension *exten,
 }
 
 /**
- * peas_extension_call_valist:
+ * peas_extension_call_valist: (skip)
  * @exten: A #PeasExtension.
  * @method_name: the name of the method that should be called.
  * @args: the arguments for the method.
@@ -212,34 +226,52 @@ peas_extension_call_valist (PeasExtension *exten,
                             va_list        args)
 {
   GICallableInfo *callable_info;
-  GITypeInfo *retval_info;
+  GITypeInfo retval_info;
   GIArgument *gargs;
   GIArgument retval;
   gpointer retval_ptr;
   gboolean ret;
+  gint n_args;
 
   g_return_val_if_fail (PEAS_IS_EXTENSION (exten), FALSE);
   g_return_val_if_fail (method_name != NULL, FALSE);
 
   callable_info = peas_gi_get_method_info (exten->priv->exten_type, method_name);
-  if (callable_info == NULL)
-    return FALSE;
+  g_return_val_if_fail (callable_info != NULL, FALSE);
 
-  gargs = g_newa (GIArgument, g_callable_info_get_n_args (callable_info));
+  n_args = g_callable_info_get_n_args (callable_info);
+  g_return_val_if_fail (n_args >= 0, FALSE);
+  gargs = g_newa (GIArgument, n_args);
   peas_gi_valist_to_arguments (callable_info, args, gargs, &retval_ptr);
 
   ret = peas_extension_callv (exten, method_name, gargs, &retval);
 
   if (retval_ptr != NULL)
     {
-      retval_info = g_callable_info_get_return_type (callable_info);
-      peas_gi_argument_to_pointer (retval_info, &retval, retval_ptr);
-      g_base_info_unref ((GIBaseInfo *) retval_info);
+      g_callable_info_load_return_type (callable_info, &retval_info);
+      peas_gi_argument_to_pointer (&retval_info, &retval, retval_ptr);
     }
 
+  g_base_info_unref ((GIBaseInfo *) callable_info);
+
   return ret;
 }
 
+/**
+ * peas_extension_callv:
+ * @exten: A #PeasExtension.
+ * @method_name: the name of the method that should be called.
+ * @args: the arguments for the method.
+ * @return_value: the return falue for the method.
+ *
+ * Call a method of the object behind @extension, using @args as arguments.
+ *
+ * See peas_extension_call() for more information.
+ *
+ * Return value: (transfer full): %TRUE on successful call.
+ *
+ * Rename to: peas_extension_call
+ */
 gboolean
 peas_extension_callv (PeasExtension *exten,
                       const gchar   *method_name,
@@ -252,5 +284,5 @@ peas_extension_callv (PeasExtension *exten,
   g_return_val_if_fail (method_name != NULL, FALSE);
 
   klass = PEAS_EXTENSION_GET_CLASS (exten);
-  return klass->call (exten, method_name, args, return_value);
+  return klass->call (exten, G_TYPE_INVALID, method_name, args, return_value);
 }
