@@ -43,6 +43,21 @@
  * @short_description: Information about a plugin.
  *
  * A #PeasPluginInfo contains all the information available about a plugin.
+ *
+ * All this information comes from the related plugin info file, whose file
+ * extension is ".plugin". Here is an example of such a plugin file, in the
+ * #GKeyFile format:
+ * |[
+ * [Plugin]
+ * Module=helloworld
+ * Name=Hello World
+ * Description=Displays "Hello World"
+ * Authors=Steve Frécinaux &lt;code@istique.net&gt;
+ * Copyright=Copyright © 2009-10 Steve Frécinaux
+ * Website=http://live.gnome.org/Libpeas
+ * Help=http://library.gnome.org/devel/libpeas/unstable/
+ * IAge=2
+ * ]|
  **/
 
 PeasPluginInfo *
@@ -78,14 +93,6 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   g_free (info);
 }
 
-/*
- * peas_plugin_info_get_type:
- *
- * Retrieves the #GType object which is associated with the #PeasPluginInfo
- * class.
- *
- * Return value: the GType associated with #PeasPluginInfo.
- **/
 GType
 peas_plugin_info_get_type (void)
 {
@@ -109,7 +116,6 @@ value_free (GValue *value)
 static void
 parse_extra_keys (PeasPluginInfo   *info,
                   GKeyFile         *plugin_file,
-                  const gchar      *section_header,
                   const gchar     **keys)
 {
   guint i;
@@ -137,13 +143,13 @@ parse_extra_keys (PeasPluginInfo   *info,
           g_str_equal (keys[i], "Builtin"))
         continue;
 
-      b = g_key_file_get_boolean (plugin_file, section_header, keys[i], &error);
+      b = g_key_file_get_boolean (plugin_file, "Plugin", keys[i], &error);
       if (b == FALSE && error != NULL)
         {
           gchar *str;
           g_error_free (error);
           error = NULL;
-          str = g_key_file_get_string (plugin_file, section_header, keys[i], NULL);
+          str = g_key_file_get_string (plugin_file, "Plugin", keys[i], NULL);
           if (str != NULL)
             {
               value = g_new0 (GValue, 1);
@@ -175,7 +181,6 @@ parse_extra_keys (PeasPluginInfo   *info,
 /*
  * _peas_plugin_info_new:
  * @filename: The filename where to read the plugin information.
- * @app_name: The application name.
  * @module_dir: The module directory.
  * @data_dir: The data directory.
  *
@@ -185,13 +190,11 @@ parse_extra_keys (PeasPluginInfo   *info,
  */
 PeasPluginInfo *
 _peas_plugin_info_new (const gchar *filename,
-                       const gchar *app_name,
                        const gchar *module_dir,
                        const gchar *data_dir)
 {
   PeasPluginInfo *info;
   GKeyFile *plugin_file = NULL;
-  gchar *section_header;
   gchar *str;
   gchar **keys;
   gint integer;
@@ -199,14 +202,11 @@ _peas_plugin_info_new (const gchar *filename,
   GError *error = NULL;
 
   g_return_val_if_fail (filename != NULL, NULL);
-  g_return_val_if_fail (app_name != NULL, NULL);
 
   info = g_new0 (PeasPluginInfo, 1);
   info->refcount = 1;
   info->file = g_strdup (filename);
 
-  section_header = g_strdup_printf ("%s Plugin", app_name);
-
   plugin_file = g_key_file_new ();
   if (!g_key_file_load_from_file (plugin_file, filename, G_KEY_FILE_NONE, NULL))
     {
@@ -214,14 +214,17 @@ _peas_plugin_info_new (const gchar *filename,
       goto error;
     }
 
-  if (!g_key_file_has_key (plugin_file, section_header, "IAge", NULL))
-    goto error;
+  if (!g_key_file_has_key (plugin_file, "Plugin", "IAge", NULL))
+    {
+      g_warning ("Could not find 'IAge' in '%s'", filename);
+      goto error;
+    }
 
-  integer = g_key_file_get_integer (plugin_file, section_header, "IAge", NULL);
+  integer = g_key_file_get_integer (plugin_file, "Plugin", "IAge", NULL);
   info->iage = integer <= 0 ? 0 : integer;
 
   /* Get module name */
-  str = g_key_file_get_string (plugin_file, section_header, "Module", NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", "Module", NULL);
 
   if ((str != NULL) && (*str != '\0'))
     {
@@ -233,15 +236,26 @@ _peas_plugin_info_new (const gchar *filename,
       goto error;
     }
 
+  /* Get Name */
+  str = g_key_file_get_locale_string (plugin_file, "Plugin",
+                                      "Name", NULL, NULL);
+  if (str)
+    info->name = str;
+  else
+    {
+      g_warning ("Could not find 'Name' in '%s'", filename);
+      goto error;
+    }
+
   /* Get the dependency list */
   info->dependencies = g_key_file_get_string_list (plugin_file,
-                                                   section_header,
+                                                   "Plugin",
                                                    "Depends", NULL, NULL);
   if (info->dependencies == NULL)
     info->dependencies = g_new0 (gchar *, 1);
 
   /* Get the loader for this plugin */
-  str = g_key_file_get_string (plugin_file, section_header, "Loader", NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", "Loader", NULL);
 
   if ((str != NULL) && (*str != '\0'))
     {
@@ -253,75 +267,60 @@ _peas_plugin_info_new (const gchar *filename,
       info->loader = g_strdup ("C");
     }
 
-  /* Get Name */
-  str = g_key_file_get_locale_string (plugin_file,
-                                      section_header, "Name", NULL, NULL);
-  if (str)
-    info->name = str;
-  else
-    {
-      g_warning ("Could not find 'Name' in '%s'", filename);
-      goto error;
-    }
-
   /* Get Description */
-  str = g_key_file_get_locale_string (plugin_file,
-                                      section_header,
+  str = g_key_file_get_locale_string (plugin_file, "Plugin",
                                       "Description", NULL, NULL);
   if (str)
     info->desc = str;
 
   /* Get Icon */
-  str = g_key_file_get_locale_string (plugin_file,
-                                      section_header, "Icon", NULL, NULL);
+  str = g_key_file_get_locale_string (plugin_file, "Plugin",
+                                      "Icon", NULL, NULL);
   if (str)
     info->icon_name = str;
 
   /* Get Authors */
-  info->authors = g_key_file_get_string_list (plugin_file,
-                                              section_header,
+  info->authors = g_key_file_get_string_list (plugin_file, "Plugin",
                                               "Authors", NULL, NULL);
 
   /* Get Copyright */
-  str = g_key_file_get_string (plugin_file,
-                               section_header, "Copyright", NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", "Copyright", NULL);
   if (str)
     info->copyright = str;
 
   /* Get Website */
-  str = g_key_file_get_string (plugin_file, section_header, "Website", NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", "Website", NULL);
   if (str)
     info->website = str;
 
   /* Get Version */
-  str = g_key_file_get_string (plugin_file, section_header, "Version", NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", "Version", NULL);
   if (str)
     info->version = str;
 
   /* Get Help URI */
-  str = g_key_file_get_string (plugin_file, section_header, OS_HELP_KEY, NULL);
+  str = g_key_file_get_string (plugin_file, "Plugin", OS_HELP_KEY, NULL);
   if (str)
     info->help_uri = str;
   else
     {
-      str = g_key_file_get_string (plugin_file, section_header, "Help", NULL);
+      str = g_key_file_get_string (plugin_file, "Plugin", "Help", NULL);
       if (str)
         info->help_uri = str;
     }
 
   /* Get Builtin */
-  b = g_key_file_get_boolean (plugin_file, section_header, "Builtin", &error);
+  b = g_key_file_get_boolean (plugin_file, "Plugin", "Builtin", &error);
   if (error != NULL)
     g_clear_error (&error);
   else
     info->builtin = b;
 
   /* Get extra keys */
-  keys = g_key_file_get_keys (plugin_file, section_header, NULL, NULL);
-  parse_extra_keys (info, plugin_file, section_header, (const gchar **) keys);
+  keys = g_key_file_get_keys (plugin_file, "Plugin", NULL, NULL);
+  parse_extra_keys (info, plugin_file, (const gchar **) keys);
   g_strfreev (keys);
 
-  g_free (section_header);
   g_key_file_free (plugin_file);
 
   info->module_dir = g_strdup (module_dir);
@@ -337,9 +336,7 @@ error:
   g_free (info->file);
   g_free (info->module_name);
   g_free (info->name);
-  g_free (info->loader);
   g_free (info);
-  g_free (section_header);
   g_key_file_free (plugin_file);
 
   return NULL;
@@ -365,9 +362,10 @@ peas_plugin_info_is_loaded (const PeasPluginInfo *info)
  * peas_plugin_info_is_available:
  * @info: A #PeasPluginInfo.
  *
- * Check if the plugin is available.  A plugin is marked as not available when
- * there is no loader available to load it, or when there has been an error
- * when trying to load it previously.
+ * Check if the plugin is available.
+ *
+ * A plugin is marked as not available when there is no loader available to
+ * load it, or when there has been an error when trying to load it previously.
  *
  * Returns: %TRUE if the plugin is available.
  */
@@ -383,7 +381,15 @@ peas_plugin_info_is_available (const PeasPluginInfo *info)
  * peas_plugin_info_is_builtin:
  * @info: A #PeasPluginInfo.
  *
- * Gets is the plugin is a builtin plugin.
+ * Check if the plugin is a builtin plugin.
+ *
+ * A builtin plugin is a plugin which cannot be enabled or disabled by the
+ * user through a plugin manager (like #PeasGtkPluginManager). Loading or
+ * unloading such plugins is the responsibility of the application alone.
+ * Most applications will usually load those plugins immediately after
+ * the initialization of the #PeasEngine.
+ *
+ * The relevant key in the plugin info file is "Builtin".
  *
  * Returns: %TRUE if the plugin is a builtin plugin, %FALSE
  * if not.
@@ -402,6 +408,12 @@ peas_plugin_info_is_builtin (const PeasPluginInfo *info)
  *
  * Gets the module name.
  *
+ * The module name will be used to find the actual plugin. The way this value
+ * will be used depends on the loader (i.e. on the language) of the plugin.
+ * This value is also used to uniquely identify a particular plugin.
+ *
+ * The relevant key in the plugin info file is "Module".
+ *
  * Returns: the module name.
  */
 const gchar *
@@ -418,6 +430,10 @@ peas_plugin_info_get_module_name (const PeasPluginInfo *info)
  *
  * Gets the module directory.
  *
+ * The module directory is the directory where the plugin file was found. This
+ * is not a value from the #GKeyFile, but rather a value provided by the
+ * #PeasEngine.
+ *
  * Returns: the module directory.
  */
 const gchar *
@@ -434,6 +450,11 @@ peas_plugin_info_get_module_dir (const PeasPluginInfo *info)
  *
  * Gets the data dir of the plugin.
  *
+ * The module data directory is the directory where a plugin should find its
+ * runtime data. This is not a value read from the #GKeyFile, but rather a
+ * value provided by the #PeasEngine, depending on where the plugin file was
+ * found.
+ *
  * Returns: the plugin's data dir.
  */
 const gchar *
@@ -444,12 +465,65 @@ peas_plugin_info_get_data_dir (const PeasPluginInfo *info)
   return info->data_dir;
 }
 
+/**
+ * peas_plugin_info_get_dependencies:
+ * @info: A #PeasPluginInfo.
+ *
+ * Gets the dependencies of the plugin.
+ *
+ * The #PeasEngine will always ensure that the dependencies of a plugin are
+ * loaded when the said plugin is loaded. It means that dependencies are
+ * loaded before the plugin, and unloaded after it. Circular dependencies of
+ * plugins lead to undefined loading order.
+ *
+ * The relevant key in the plugin info file is "Depends".
+ *
+ * Returns: (transfer none): the plugin's dependencies.
+ */
+const gchar **
+peas_plugin_info_get_dependencies (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, NULL);
+
+  return (const gchar **) info->dependencies;
+}
+
+/**
+ * peas_plugin_info_has_dependency:
+ * @info: A #PeasPluginInfo.
+ * @module_name: The name of the plugin to check.
+ *
+ * Check if the plugin depends on another plugin.
+ *
+ * Returns: whether the plugin depends on the plugin @module_name.
+ */
+gboolean
+peas_plugin_info_has_dependency (const PeasPluginInfo *info,
+                                 const gchar          *module_name)
+{
+  guint i;
+
+  g_return_val_if_fail (info != NULL, FALSE);
+  g_return_val_if_fail (module_name != NULL, FALSE);
+
+  for (i = 0; info->dependencies[i] != NULL; i++)
+    if (g_ascii_strcasecmp (module_name, info->dependencies[i]) == 0)
+      return TRUE;
+
+  return FALSE;
+}
+
+
 /**
  * peas_plugin_info_get_name:
  * @info: A #PeasPluginInfo.
  *
  * Gets the name of the plugin.
  *
+ * The name of a plugin should be a nice short string to be presented in UIs.
+ *
+ * The relevant key in the plugin info file is "Name".
+ *
  * Returns: the plugin's name.
  */
 const gchar *
@@ -466,6 +540,11 @@ peas_plugin_info_get_name (const PeasPluginInfo *info)
  *
  * Gets the description of the plugin.
  *
+ * The description of the plugin should be a string presenting the purpose of
+ * the plugin. It will typically be presented in a plugin's about box.
+ *
+ * The relevant key in the plugin info file is "Description".
+ *
  * Returns: the plugin's description.
  */
 const gchar *
@@ -482,6 +561,11 @@ peas_plugin_info_get_description (const PeasPluginInfo *info)
  *
  * Gets the icon name of the plugin.
  *
+ * The icon of the plugin will be presented in the plugin manager UI. If no
+ * icon is specified, the default green puzzle icon will be used.
+ *
+ * The relevant key in the plugin info file is "Icon".
+ *
  * Returns: the plugin's icon name.
  */
 const gchar *
@@ -503,7 +587,9 @@ peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
  *
  * Gets a NULL-terminated array of strings with the authors of the plugin.
  *
- * Returns: the plugin's author list.
+ * The relevant key in the plugin info file is "Authors".
+ *
+ * Returns: (transfer none): the plugin's author list.
  */
 const gchar **
 peas_plugin_info_get_authors (const PeasPluginInfo *info)
@@ -519,6 +605,8 @@ peas_plugin_info_get_authors (const PeasPluginInfo *info)
  *
  * Gets the website of the plugin.
  *
+ * The relevant key in the plugin info file is "Website".
+ *
  * Returns: the plugin's associated website.
  */
 const gchar *
@@ -535,6 +623,8 @@ peas_plugin_info_get_website (const PeasPluginInfo *info)
  *
  * Gets the copyright of the plugin.
  *
+ * The relevant key in the plugin info file is "Copyright".
+ *
  * Returns: the plugin's copyright information.
  */
 const gchar *
@@ -551,6 +641,8 @@ peas_plugin_info_get_copyright (const PeasPluginInfo *info)
  *
  * Gets the version of the plugin.
  *
+ * The relevant key in the plugin info file is "Version".
+ *
  * Returns: the plugin's version.
  */
 const gchar *
@@ -567,6 +659,14 @@ peas_plugin_info_get_version (const PeasPluginInfo *info)
  *
  * Gets the help URI of the plugin.
  *
+ * The Help URI of a plugin will typically be presented by the plugin manager
+ * as a "Help" button linking to the URI. It can either be a HTTP URL on some
+ * website or a ghelp: URI if a Gnome help page is available for the plugin.
+ *
+ * The relevant key in the plugin info file is "Help". Other platform-specific
+ * keys exist for platform-specific help files. Those are "Help-GNOME",
+ * "Help-Windows" and "Help-MacOS-X".
+ *
  * Returns: the plugin's help URI.
  */
 const gchar *
@@ -583,6 +683,8 @@ peas_plugin_info_get_help_uri (const PeasPluginInfo *info)
  *
  * Gets the interface age of the plugin.
  *
+ * The relevant key in the plugin info file is "IAge".
+ *
  * Returns: the interface age of the plugin or %0 if not known.
  **/
 gint
@@ -600,8 +702,8 @@ peas_plugin_info_get_iage (const PeasPluginInfo *info)
  * Gets a hash table of string keys present and #GValue values,
  * present in the plugin information file, but not handled
  * by libpeas. Note that libpeas only handles booleans and
- * strings, and that strings that are recognised as booleans,
- * as done by #g_key_file_get_boolean, will be of boolean type.
+ * strings, and that strings that are recognized as booleans,
+ * as done by g_key_file_get_boolean(), will be of boolean type.
  *
  * Returns: a #GHashTable of string keys and #GValue values. Do
  * not free or destroy any data in this hashtable.
